@@ -1011,6 +1011,8 @@ short valid_domain (const char *dname)
         (dname[0] == '-') ||
         (dname[len - 1] == '.') ||
         (dname[0] == '.') ||
+        strstr(dname, ".-") ||
+        strstr(dname, "-.") ||
         strstr(dname, "..")) {
         return 0;
     }
@@ -1091,6 +1093,7 @@ int netcfg_get_hostname(struct debconfclient *client, char *template, char *host
 int netcfg_get_domain(struct debconfclient *client,  char domain[])
 {
     int ret;
+    char buf[6];
 
     if (have_domain == 1)
     {
@@ -1100,23 +1103,35 @@ int netcfg_get_domain(struct debconfclient *client,  char domain[])
         return 0;
     }
 
-    debconf_input (client, "high", "netcfg/get_domain");
-    ret = debconf_go(client);
+    while(1) {
+        debconf_input(client, "high", "netcfg/get_domain");
+        ret = debconf_go(client);
+        if (ret)
+            return ret;
 
-    if (ret)
-        return ret;
-
-    debconf_get (client, "netcfg/get_domain");
-
-    *domain = '\0';
-    if (!empty_str(client->value)) {
-        const char *start = client->value;
-        while (*start == '.')
-            ++start; /* trim leading dots */
-        strncpy(domain, start, MAXHOSTNAMELEN);
+        debconf_get(client, "netcfg/get_domain");
+        *domain = '\0';
+        if (empty_str(client->value))
+            return 0;
+        else {
+            const char *start = client->value;
+            while (*start == '.')
+                ++start; /* trim leading dots */
+            if (valid_domain(start)) {
+                strncpy(domain, start, MAXHOSTNAMELEN);
+                return 0;
+            }
+            di_info("%s is an invalid domain name", start);
+            debconf_subst(client, "netcfg/invalid_domain",
+                          "domain", start);
+            snprintf(buf, sizeof(buf), "%i", MAXHOSTNAMELEN);
+            debconf_subst(client, "netcfg/invalid_domain",
+                          "maxhostnamelen", buf);
+            debconf_input(client, "high", "netcfg/invalid_domain");
+            debconf_go(client);
+            debconf_set(client, "netcfg/get_domain", "");
+        }
     }
-
-    return 0;
 }
 
 void netcfg_write_loopback (void)
@@ -1384,11 +1399,12 @@ void reap_old_files (void)
 /* Convert a space-separated list of nameservers in a single string (as might
  * be entered into, say, debconf), and store them in the interface.
  */
-void netcfg_nameservers_to_array(const char *nameservers, struct netcfg_interface *interface)
+int netcfg_nameservers_to_array(const char *nameservers, struct netcfg_interface *interface)
 {
     char *save, *ptr, *ns;
     unsigned int i;
     union inX_addr addr;
+    int ok = 1;
 
     if (nameservers) {
         save = ptr = strdup(nameservers);
@@ -1411,7 +1427,8 @@ void netcfg_nameservers_to_array(const char *nameservers, struct netcfg_interfac
                         af = AF_INET6;
                     } else {
                         af = -1;
-                        fprintf(stderr, "Failed to parse %s as an IP address", ns);
+                        fprintf(stderr, "Failed to parse %s as an IP address\n", ns);
+                        ok = 0;
                     }
                 }
 
@@ -1430,12 +1447,14 @@ void netcfg_nameservers_to_array(const char *nameservers, struct netcfg_interfac
         /* Empty out all the nameserver strings */
         for (i = 0; i < NETCFG_NAMESERVERS_MAX; i++) *(interface->nameservers[i]) = '\0';
     }
+    return ok;
 }
 
-int netcfg_get_nameservers (struct debconfclient *client, char **nameservers, char *default_nameservers)
+int netcfg_get_nameservers (struct debconfclient *client, char **nameservers, char *default_nameservers,
+                            struct netcfg_interface *interface)
 {
     char *ptr;
-    int ret;
+    int ret, ok = 0;
 
     debconf_get(client,"netcfg/get_nameservers");
     if (*nameservers)
@@ -1448,14 +1467,24 @@ int netcfg_get_nameservers (struct debconfclient *client, char **nameservers, ch
         ptr = "";
     debconf_set(client, "netcfg/get_nameservers", ptr);
 
-    debconf_input(client, "critical", "netcfg/get_nameservers");
-    ret = debconf_go(client);
+    while (!ok) {
+        debconf_input(client, "critical", "netcfg/get_nameservers");
+        ret = debconf_go(client);
 
-    if (ret)
-        return ret;
+        if (ret)
+            return ret;
 
-    debconf_get(client, "netcfg/get_nameservers");
-    ptr = client->value;
+        debconf_get(client, "netcfg/get_nameservers");
+        ptr = client->value;
+        ok = netcfg_nameservers_to_array(ptr, interface);
+
+        if (!ok) {
+            debconf_capb(client);
+            debconf_input (client, "critical", "netcfg/bad_ipaddress");
+            debconf_go (client);
+            debconf_capb(client, "backup");
+        }
+    }
 
     if (*nameservers)
         free(*nameservers);
